searchData={"items":[{"type":"module","title":"Plug.Crypto","doc":"Namespace and module for crypto-related functionality.\n\nFor low-level functionality, see `Plug.Crypto.KeyGenerator`,\n`Plug.Crypto.MessageEncryptor`, and `Plug.Crypto.MessageVerifier`.","ref":"Plug.Crypto.html"},{"type":"function","title":"Plug.Crypto.decrypt/4","doc":"Decrypts the original data from the token and verifies its integrity.","ref":"Plug.Crypto.html#decrypt/4"},{"type":"function","title":"Options - Plug.Crypto.decrypt/4","doc":"* `:max_age` - verifies the token only if it has been generated\n    \"max age\" ago in seconds. A reasonable value is 1 day (86400\n    seconds)\n  * `:key_iterations` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 1000\n  * `:key_length` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 32\n  * `:key_digest` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to `:sha256`","ref":"Plug.Crypto.html#decrypt/4-options"},{"type":"function","title":"Plug.Crypto.encrypt/4","doc":"Encodes, encrypts, and signs data into a token you can send to clients.\n\n    Plug.Crypto.encrypt(conn.secret_key_base, \"user-secret\", {:elixir, :terms})\n\nA key will be derived from the secret key base and the given user secret.\nThe key will also be cached for performance reasons on future calls.","ref":"Plug.Crypto.html#encrypt/4"},{"type":"function","title":"Options - Plug.Crypto.encrypt/4","doc":"* `:key_iterations` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 1000\n  * `:key_length` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 32\n  * `:key_digest` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to `:sha256`\n  * `:signed_at` - set the timestamp of the token in seconds.\n    Defaults to `System.os_time(:millisecond)`\n  * `:max_age` - the default maximum age of the token. Defaults to\n    `86400` seconds (1 day) and it may be overridden on `decrypt/4`.","ref":"Plug.Crypto.html#encrypt/4-options"},{"type":"function","title":"Plug.Crypto.mask/2","doc":"Masks the token on the left with the token on the right.\n\nBoth tokens are required to have the same size.","ref":"Plug.Crypto.html#mask/2"},{"type":"function","title":"Plug.Crypto.masked_compare/3","doc":"Compares the two binaries (one being masked) in constant-time to avoid\ntiming attacks.\n\nIt is assumed the right token is masked according to the given mask.","ref":"Plug.Crypto.html#masked_compare/3"},{"type":"function","title":"Plug.Crypto.non_executable_binary_to_term/2","doc":"A restricted version of `:erlang.binary_to_term/2` that forbids\n*executable* terms, such as anonymous functions.\n\nThe `opts` are given to the underlying `:erlang.binary_to_term/2`\ncall, with an empty list as a default.\n\nBy default this function does not restrict atoms, as an atom\ninterned in one node may not yet have been interned on another\n(except for releases, which preload all code).\n\nIf you want to avoid atoms from being created, then you can pass\n`[:safe]` as options, as that will also enable the safety mechanisms\nfrom `:erlang.binary_to_term/2` itself.","ref":"Plug.Crypto.html#non_executable_binary_to_term/2"},{"type":"function","title":"Plug.Crypto.prune_args_from_stacktrace/1","doc":"Prunes the stacktrace to remove any argument trace.\n\nThis is useful when working with functions that receives secrets\nand we want to make sure those secrets do not leak on error messages.","ref":"Plug.Crypto.html#prune_args_from_stacktrace/1"},{"type":"function","title":"Plug.Crypto.secure_compare/2","doc":"Compares the two binaries in constant-time to avoid timing attacks.\n\nSee: http://codahale.com/a-lesson-in-timing-attacks/","ref":"Plug.Crypto.html#secure_compare/2"},{"type":"function","title":"Plug.Crypto.sign/4","doc":"Encodes and signs data into a token you can send to clients.\n\n    Plug.Crypto.sign(conn.secret_key_base, \"user-secret\", {:elixir, :terms})\n\nA key will be derived from the secret key base and the given user secret.\nThe key will also be cached for performance reasons on future calls.","ref":"Plug.Crypto.html#sign/4"},{"type":"function","title":"Options - Plug.Crypto.sign/4","doc":"* `:key_iterations` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 1000\n  * `:key_length` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 32\n  * `:key_digest` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to `:sha256`\n  * `:signed_at` - set the timestamp of the token in seconds.\n    Defaults to `System.os_time(:millisecond)`\n  * `:max_age` - the default maximum age of the token. Defaults to\n    `86400` seconds (1 day) and it may be overridden on `verify/4`.","ref":"Plug.Crypto.html#sign/4-options"},{"type":"function","title":"Plug.Crypto.verify/4","doc":"Decodes the original data from the token and verifies its integrity.","ref":"Plug.Crypto.html#verify/4"},{"type":"function","title":"Examples - Plug.Crypto.verify/4","doc":"In this scenario we will create a token, sign it, then provide it to a client\napplication. The client will then use this token to authenticate requests for\nresources from the server. See `Plug.Crypto` summary for more info about\ncreating tokens.\n\n    iex> user_id    = 99\n    iex> secret     = \"kjoy3o1zeidquwy1398juxzldjlksahdk3\"\n    iex> user_salt  = \"user salt\"\n    iex> token      = Plug.Crypto.sign(secret, user_salt, user_id)\n\nThe mechanism for passing the token to the client is typically through a\ncookie, a JSON response body, or HTTP header. For now, assume the client has\nreceived a token it can use to validate requests for protected resources.\n\nWhen the server receives a request, it can use `verify/4` to determine if it\nshould provide the requested resources to the client:\n\n    iex> Plug.Crypto.verify(secret, user_salt, token, max_age: 86400)\n    {:ok, 99}\n\nIn this example, we know the client sent a valid token because `verify/4`\nreturned a tuple of type `{:ok, user_id}`. The server can now proceed with\nthe request.\n\nHowever, if the client had sent an expired or otherwise invalid token\n`verify/4` would have returned an error instead:\n\n    iex> Plug.Crypto.verify(secret, user_salt, expired, max_age: 86400)\n    {:error, :expired}\n\n    iex> Plug.Crypto.verify(secret, user_salt, invalid, max_age: 86400)\n    {:error, :invalid}","ref":"Plug.Crypto.html#verify/4-examples"},{"type":"function","title":"Options - Plug.Crypto.verify/4","doc":"* `:max_age` - verifies the token only if it has been generated\n    \"max age\" ago in seconds. Defaults to the max age signed in the\n    token (86400)\n  * `:key_iterations` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 1000\n  * `:key_length` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 32\n  * `:key_digest` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to `:sha256`","ref":"Plug.Crypto.html#verify/4-options"},{"type":"module","title":"Plug.Crypto.KeyGenerator","doc":"`KeyGenerator` implements PBKDF2 (Password-Based Key Derivation Function 2),\npart of PKCS #5 v2.0 (Password-Based Cryptography Specification).\n\nIt can be used to derive a number of keys for various purposes from a given\nsecret. This lets applications have a single secure secret, but avoid reusing\nthat key in multiple incompatible contexts.\n\nThe returned key is a binary. You may invoke functions in the `Base` module,\nsuch as `Base.url_encode64/2`, to convert this binary into a textual\nrepresentation.\n\nSee http://tools.ietf.org/html/rfc2898#section-5.2","ref":"Plug.Crypto.KeyGenerator.html"},{"type":"function","title":"Plug.Crypto.KeyGenerator.generate/3","doc":"Returns a derived key suitable for use.","ref":"Plug.Crypto.KeyGenerator.html#generate/3"},{"type":"function","title":"Options - Plug.Crypto.KeyGenerator.generate/3","doc":"* `:iterations` - defaults to 1000 (increase to at least 2^16 if used for passwords);\n  * `:length`     - a length in octets for the derived key. Defaults to 32;\n  * `:digest`     - an hmac function to use as the pseudo-random function. Defaults to `:sha256`;\n  * `:cache`      - an ETS table name to be used as cache.\n    Only use an ETS table as cache if the secret and salt is a bound set of values.\n    For example: `:ets.new(:your_name, [:named_table, :public, read_concurrency: true])`","ref":"Plug.Crypto.KeyGenerator.html#generate/3-options"},{"type":"module","title":"Plug.Crypto.MessageEncryptor","doc":"`MessageEncryptor` is a simple way to encrypt values which get stored\nsomewhere you don't trust.\n\nThe encrypted key, initialization vector, cipher text, and cipher tag\nare base64url encoded and returned to you.\n\nThis can be used in situations similar to the `Plug.Crypto.MessageVerifier`,\nbut where you don't want users to be able to determine the value of the payload.\n\nThe current algorithm used is XChaCha20-Poly1305.","ref":"Plug.Crypto.MessageEncryptor.html"},{"type":"module","title":"Example - Plug.Crypto.MessageEncryptor","doc":"iex> secret_key_base = \"072d1e0157c008193fe48a670cce031faa4e...\"\n    ...> encrypted_cookie_salt = \"encrypted cookie\"\n    ...> secret = KeyGenerator.generate(secret_key_base, encrypted_cookie_salt)\n    ...>\n    ...> data = \"José\"\n    ...> encrypted = MessageEncryptor.encrypt(data, secret, \"UNUSED\")\n    ...> MessageEncryptor.decrypt(encrypted, secret, \"UNUSED\")\n    {:ok, \"José\"}","ref":"Plug.Crypto.MessageEncryptor.html#module-example"},{"type":"function","title":"Plug.Crypto.MessageEncryptor.decrypt/4","doc":"Decrypts a message using authenticated encryption.","ref":"Plug.Crypto.MessageEncryptor.html#decrypt/4"},{"type":"function","title":"Plug.Crypto.MessageEncryptor.encrypt/4","doc":"Encrypts a message using authenticated encryption.\n\nThe `sign_secret` is currently only used on decryption\nfor backwards compatibility.\n\nA custom authentication message can be provided.\nIt defaults to \"A128GCM\" for backwards compatibility.","ref":"Plug.Crypto.MessageEncryptor.html#encrypt/4"},{"type":"module","title":"Plug.Crypto.MessageVerifier","doc":"`MessageVerifier` makes it easy to generate and verify messages\nwhich are signed to prevent tampering.\n\nFor example, the cookie store uses this verifier to send data\nto the client. The data can be read by the client, but cannot be\ntampered with.\n\nThe message and its verification are base64url encoded and returned\nto you.\n\nThe current algorithm used is HMAC-SHA, with SHA256, SHA384, and\nSHA512 as supported digest types.","ref":"Plug.Crypto.MessageVerifier.html"},{"type":"function","title":"Plug.Crypto.MessageVerifier.sign/3","doc":"Signs a message according to the given secret.","ref":"Plug.Crypto.MessageVerifier.html#sign/3"},{"type":"function","title":"Plug.Crypto.MessageVerifier.verify/2","doc":"Decodes and verifies the encoded binary was not tampered with.","ref":"Plug.Crypto.MessageVerifier.html#verify/2"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.1.1 (2025-04-03) - Changelog","doc":"* Fall back `hash_equals` when missing OpenSSL support","ref":"changelog.html#v2-1-1-2025-04-03"},{"type":"extras","title":"v2.1.0 (2024-05-02) - Changelog","doc":"* Use `System.os_time/1` as the token signing date, since tokens are meant to be shared across machines","ref":"changelog.html#v2-1-0-2024-05-02"},{"type":"extras","title":"v2.0.0 (2023-10-06) - Changelog","doc":"* Update Elixir requirement to v1.11+ and require Erlang/OTP 23.\n  * Encryption now uses XChaCha20-Poly1305, which is safer, faster, and generates smaller payloads. This means data encrypted with Plug.Crypto v2.0 cannot be decrypted on Plug.Crypto v1.x. However, Plug.Crypto v2.0 can still decrypt data from Plug.Crypto v1.0.\n  * Optimize `secure_compare`, `masked_compare`, and key generator algorithms by relying on `:crypto` code when using more recent Erlang/OTP versions.","ref":"changelog.html#v2-0-0-2023-10-06"},{"type":"extras","title":"v1.2.5 (2023-03-10) - Changelog","doc":"* Allow AAD to be given as iolist","ref":"changelog.html#v1-2-5-2023-03-10"},{"type":"extras","title":"v1.2.4 (2023-03-10) - Changelog","doc":"* Allow AAD to be given as argument on message encryptor","ref":"changelog.html#v1-2-4-2023-03-10"},{"type":"extras","title":"v1.2.3 (2022-08-19) - Changelog","doc":"* Remove warnings on Elixir v1.14","ref":"changelog.html#v1-2-3-2022-08-19"},{"type":"extras","title":"v1.2.2 (2021-03-25) - Changelog","doc":"* Remove warnings on Elixir v1.12","ref":"changelog.html#v1-2-2-2021-03-25"},{"type":"extras","title":"v1.2.1 (2021-02-17) - Changelog","doc":"* Add support for Erlang/OTP 24","ref":"changelog.html#v1-2-1-2021-02-17"},{"type":"extras","title":"v1.2.0 (2020-10-07) - Changelog","doc":"* Update Elixir requirement to Elixir 1.7+.\n  * Fixed a bug that allowed to sign and encrypt stuff with `nil` secret key base and salt.","ref":"changelog.html#v1-2-0-2020-10-07"},{"type":"extras","title":"v1.1.2 (2020-02-16) - Changelog","doc":"* Do not key derive empty salts (default to no salt instead).","ref":"changelog.html#v1-1-2-2020-02-16"},{"type":"extras","title":"v1.1.1 (2020-02-14) - Changelog","doc":"* Do not expose encryption with salt API.\n  * Allow default `:max_age` to be set when signing/encrypting.","ref":"changelog.html#v1-1-1-2020-02-14"},{"type":"extras","title":"v1.1.0 (2020-02-11) - Changelog","doc":"* Add high-level `Plug.Crypto.sign/verify` and `Plug.Crypto.encrypt/decrypt`.","ref":"changelog.html#v1-1-0-2020-02-11"},{"type":"extras","title":"v1.0.0 (2018-10-03) - Changelog","doc":"* Split up the `plug_crypto` project from Plug as per [elixir-lang/plug#766](https://github.com/elixir-plug/plug/issues/766).","ref":"changelog.html#v1-0-0-2018-10-03"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}